summaryrefslogtreecommitdiffstats
path: root/content/browser
diff options
context:
space:
mode:
authorsammc@chromium.org <sammc@chromium.org@0039d316-1c4b-4281-b951-d872f2087c98>2014-06-25 06:09:25 +0000
committersammc@chromium.org <sammc@chromium.org@0039d316-1c4b-4281-b951-d872f2087c98>2014-06-25 06:09:25 +0000
commitc52a141be92a438e61a5c1784f2ac10c2c5adb9e (patch)
treea323e3c770b343687523d156958450bdbeab940e /content/browser
parente07862b492e587392feb466d2604c34e99c7d314 (diff)
downloadchromium_src-c52a141be92a438e61a5c1784f2ac10c2c5adb9e.zip
chromium_src-c52a141be92a438e61a5c1784f2ac10c2c5adb9e.tar.gz
chromium_src-c52a141be92a438e61a5c1784f2ac10c2c5adb9e.tar.bz2
Support exposing Mojo services between render frames, render threads, and their respective hosts.
This introduces ServiceRegistry as an abstraction around providing services to and accessing services from a remote peer. In particular, this adds peered service registries to RenderProcessHost and RenderThread, and to RenderFrameHost and RenderFrame - the RenderFrame setup is implemented using the RenderProcessHost/RenderThread ServiceRegistry pair. This replaces the existing WebUI handle setup by adding a webUI controller service to the frame host registry and a corresponding request for the webUI controller service to the frame registry. BUG=386155 Committed: https://src.chromium.org/viewvc/chrome?view=rev&revision=279557 Review URL: https://codereview.chromium.org/285333003 git-svn-id: svn://svn.chromium.org/chrome/trunk/src@279623 0039d316-1c4b-4281-b951-d872f2087c98
Diffstat (limited to 'content/browser')
-rw-r--r--content/browser/BUILD.gn2
-rw-r--r--content/browser/frame_host/render_frame_host_impl.cc17
-rw-r--r--content/browser/frame_host/render_frame_host_impl.h4
-rw-r--r--content/browser/mojo/mojo_application_host.cc13
-rw-r--r--content/browser/mojo/mojo_application_host.h32
-rw-r--r--content/browser/renderer_host/render_process_host_impl.cc23
-rw-r--r--content/browser/renderer_host/render_process_host_impl.h21
-rw-r--r--content/browser/renderer_host/render_view_host_impl.cc23
-rw-r--r--content/browser/renderer_host/render_view_host_impl.h1
-rw-r--r--content/browser/webui/web_ui_mojo_browsertest.cc30
10 files changed, 73 insertions, 93 deletions
diff --git a/content/browser/BUILD.gn b/content/browser/BUILD.gn
index 325401f..7ec025c 100644
--- a/content/browser/BUILD.gn
+++ b/content/browser/BUILD.gn
@@ -86,7 +86,7 @@ source_set("browser") {
"//cc",
"//cc:cc_surfaces",
"//mojo/public/cpp/bindings",
- "//mojo/public/interfaces/service_provider:service_provider",
+ "//mojo/public/interfaces/interface_provider",
"//mojo/public/js/bindings",
"//net:http_server",
"//third_party/icu",
diff --git a/content/browser/frame_host/render_frame_host_impl.cc b/content/browser/frame_host/render_frame_host_impl.cc
index bfeee98..9ffcbef 100644
--- a/content/browser/frame_host/render_frame_host_impl.cc
+++ b/content/browser/frame_host/render_frame_host_impl.cc
@@ -18,6 +18,7 @@
#include "content/browser/frame_host/render_frame_proxy_host.h"
#include "content/browser/renderer_host/input/input_router.h"
#include "content/browser/renderer_host/input/timeout_monitor.h"
+#include "content/browser/renderer_host/render_process_host_impl.h"
#include "content/browser/renderer_host/render_view_host_impl.h"
#include "content/browser/renderer_host/render_widget_host_impl.h"
#include "content/browser/transition_request_manager.h"
@@ -25,6 +26,7 @@
#include "content/common/frame_messages.h"
#include "content/common/input_messages.h"
#include "content/common/inter_process_time_ticks_converter.h"
+#include "content/common/render_frame_setup.mojom.h"
#include "content/common/swapped_out_messages.h"
#include "content/public/browser/browser_thread.h"
#include "content/public/browser/content_browser_client.h"
@@ -162,6 +164,16 @@ RenderFrameHostImpl::RenderFrameHostImpl(
g_routing_id_frame_map.Get().insert(std::make_pair(
RenderFrameHostID(GetProcess()->GetID(), routing_id_),
this));
+
+ if (GetProcess()->GetServiceRegistry()) {
+ RenderFrameSetupPtr setup;
+ GetProcess()->GetServiceRegistry()->GetRemoteInterface(&setup);
+ mojo::IInterfaceProviderPtr service_provider;
+ setup->GetServiceProviderForFrame(routing_id_,
+ mojo::Get(&service_provider));
+ service_registry_.BindRemoteServiceProvider(
+ service_provider.PassMessagePipe());
+ }
}
RenderFrameHostImpl::~RenderFrameHostImpl() {
@@ -242,6 +254,11 @@ RenderViewHost* RenderFrameHostImpl::GetRenderViewHost() {
return render_view_host_;
}
+ServiceRegistry* RenderFrameHostImpl::GetServiceRegistry() {
+ static_cast<RenderProcessHostImpl*>(GetProcess())->EnsureMojoActivated();
+ return &service_registry_;
+}
+
bool RenderFrameHostImpl::Send(IPC::Message* message) {
if (IPC_MESSAGE_ID_CLASS(message->type()) == InputMsgStart) {
return render_view_host_->input_router()->SendInput(
diff --git a/content/browser/frame_host/render_frame_host_impl.h b/content/browser/frame_host/render_frame_host_impl.h
index b554511..092b630 100644
--- a/content/browser/frame_host/render_frame_host_impl.h
+++ b/content/browser/frame_host/render_frame_host_impl.h
@@ -14,6 +14,7 @@
#include "base/strings/string16.h"
#include "base/time/time.h"
#include "content/common/content_export.h"
+#include "content/common/mojo/service_registry_impl.h"
#include "content/public/browser/render_frame_host.h"
#include "content/public/common/javascript_message_type.h"
#include "content/public/common/page_transition_types.h"
@@ -66,6 +67,7 @@ class CONTENT_EXPORT RenderFrameHostImpl : public RenderFrameHost {
const base::string16& javascript,
const JavaScriptResultCallback& callback) OVERRIDE;
virtual RenderViewHost* GetRenderViewHost() OVERRIDE;
+ virtual ServiceRegistry* GetServiceRegistry() OVERRIDE;
// IPC::Sender
virtual bool Send(IPC::Message* msg) OVERRIDE;
@@ -302,6 +304,8 @@ class CONTENT_EXPORT RenderFrameHostImpl : public RenderFrameHost {
// When the last BeforeUnload message was sent.
base::TimeTicks send_before_unload_start_time_;
+ ServiceRegistryImpl service_registry_;
+
base::WeakPtrFactory<RenderFrameHostImpl> weak_ptr_factory_;
DISALLOW_COPY_AND_ASSIGN(RenderFrameHostImpl);
diff --git a/content/browser/mojo/mojo_application_host.cc b/content/browser/mojo/mojo_application_host.cc
index 90187b9..2491252 100644
--- a/content/browser/mojo/mojo_application_host.cc
+++ b/content/browser/mojo/mojo_application_host.cc
@@ -30,7 +30,7 @@ MojoApplicationHost::~MojoApplicationHost() {
}
bool MojoApplicationHost::Init() {
- DCHECK(!child_service_provider_.get()) << "Already initialized!";
+ DCHECK(!client_handle_.is_valid()) << "Already initialized!";
mojo::embedder::PlatformChannelPair channel_pair;
@@ -43,8 +43,7 @@ bool MojoApplicationHost::Init() {
// Forward this to the client once we know its process handle.
client_handle_ = channel_pair.PassClientHandle();
- child_service_provider_.reset(
- BindToPipe(new ServiceProviderImpl(), message_pipe.Pass()));
+ service_registry_.BindRemoteServiceProvider(message_pipe.Pass());
return true;
}
@@ -60,12 +59,4 @@ bool MojoApplicationHost::Activate(IPC::Sender* sender,
return did_activate_;
}
-void MojoApplicationHost::ServiceProviderImpl::ConnectToService(
- const mojo::String& service_url,
- const mojo::String& service_name,
- mojo::ScopedMessagePipeHandle handle,
- const mojo::String& requestor_url) {
- // TODO(darin): Provide something meaningful here.
-}
-
} // namespace content
diff --git a/content/browser/renderer_host/render_process_host_impl.cc b/content/browser/renderer_host/render_process_host_impl.cc
index 7b5cd1c..b67ecae 100644
--- a/content/browser/renderer_host/render_process_host_impl.cc
+++ b/content/browser/renderer_host/render_process_host_impl.cc
@@ -69,7 +69,6 @@
#include "content/browser/media/midi_host.h"
#include "content/browser/message_port_message_filter.h"
#include "content/browser/mime_registry_message_filter.h"
-#include "content/browser/mojo/mojo_application_host.h"
#include "content/browser/plugin_service_impl.h"
#include "content/browser/profiler_message_filter.h"
#include "content/browser/push_messaging_message_filter.h"
@@ -141,7 +140,6 @@
#include "ipc/ipc_logging.h"
#include "ipc/ipc_switches.h"
#include "media/base/media_switches.h"
-#include "mojo/common/common_type_converters.h"
#include "net/url_request/url_request_context_getter.h"
#include "ppapi/shared_impl/ppapi_switches.h"
#include "third_party/skia/include/core/SkBitmap.h"
@@ -448,6 +446,7 @@ RenderProcessHostImpl::RenderProcessHostImpl(
is_self_deleted_(false),
#endif
pending_views_(0),
+ mojo_application_host_(new MojoApplicationHost),
mojo_activation_required_(false),
visible_widgets_(0),
backgrounded_(true),
@@ -597,7 +596,6 @@ bool RenderProcessHostImpl::Init() {
BrowserThread::GetMessageLoopProxyForThread(BrowserThread::IO).get());
// Setup the Mojo channel.
- mojo_application_host_.reset(new MojoApplicationHost());
mojo_application_host_->Init();
// Call the embedder first so that their IPC filters have priority.
@@ -908,6 +906,12 @@ void RenderProcessHostImpl::NotifyTimezoneChange() {
Send(new ViewMsg_TimezoneChange());
}
+ServiceRegistry* RenderProcessHostImpl::GetServiceRegistry() {
+ if (!mojo_application_host_)
+ return NULL;
+ return mojo_application_host_->service_registry();
+}
+
void RenderProcessHostImpl::AddRoute(
int32 routing_id,
IPC::Listener* listener) {
@@ -1933,7 +1937,8 @@ void RenderProcessHostImpl::ProcessDied(bool already_dead) {
iter.Advance();
}
- mojo_application_host_.reset();
+ mojo_application_host_.reset(new MojoApplicationHost);
+ mojo_activation_required_ = false;
// It's possible that one of the calls out to the observers might have caused
// this object to be no longer needed.
@@ -2253,17 +2258,9 @@ void RenderProcessHostImpl::DecrementWorkerRefCount() {
Cleanup();
}
-void RenderProcessHostImpl::ConnectTo(
- const base::StringPiece& service_name,
- mojo::ScopedMessagePipeHandle handle) {
+void RenderProcessHostImpl::EnsureMojoActivated() {
mojo_activation_required_ = true;
MaybeActivateMojo();
-
- mojo_application_host_->service_provider()->ConnectToService(
- mojo::String::From(service_name),
- std::string(),
- handle.Pass(),
- mojo::String());
}
void RenderProcessHostImpl::OnAllocateGpuMemoryBuffer(uint32 width,
diff --git a/content/browser/renderer_host/render_process_host_impl.h b/content/browser/renderer_host/render_process_host_impl.h
index 61f9910..36f8f9a 100644
--- a/content/browser/renderer_host/render_process_host_impl.h
+++ b/content/browser/renderer_host/render_process_host_impl.h
@@ -15,8 +15,10 @@
#include "base/timer/timer.h"
#include "content/browser/child_process_launcher.h"
#include "content/browser/dom_storage/session_storage_namespace_impl.h"
+#include "content/browser/mojo/mojo_application_host.h"
#include "content/browser/power_monitor_message_broadcaster.h"
#include "content/common/content_export.h"
+#include "content/common/mojo/service_registry_impl.h"
#include "content/public/browser/gpu_data_manager_observer.h"
#include "content/public/browser/render_process_host.h"
#include "ipc/ipc_channel_proxy.h"
@@ -45,13 +47,11 @@ class AudioRendererHost;
class BrowserDemuxerAndroid;
class GpuMessageFilter;
class MessagePortMessageFilter;
-class MojoApplicationHost;
#if defined(ENABLE_WEBRTC)
class P2PSocketDispatcherHost;
#endif
class PeerConnectionTrackerHost;
class RendererMainThread;
-class RenderProcessHostMojoImpl;
class RenderWidgetHelper;
class RenderWidgetHost;
class RenderWidgetHostImpl;
@@ -143,6 +143,7 @@ class CONTENT_EXPORT RenderProcessHostImpl
virtual void ResumeDeferredNavigation(const GlobalRequestID& request_id)
OVERRIDE;
virtual void NotifyTimezoneChange() OVERRIDE;
+ virtual ServiceRegistry* GetServiceRegistry() OVERRIDE;
// IPC::Sender via RenderProcessHost.
virtual bool Send(IPC::Message* msg) OVERRIDE;
@@ -250,23 +251,13 @@ class CONTENT_EXPORT RenderProcessHostImpl
void IncrementWorkerRefCount();
void DecrementWorkerRefCount();
- // Establish a connection to a renderer-provided service. See
- // content/common/mojo/mojo_service_names.h for a list of services.
- void ConnectTo(const base::StringPiece& service_name,
- mojo::ScopedMessagePipeHandle handle);
-
- template <typename Interface>
- void ConnectTo(const base::StringPiece& service_name,
- mojo::InterfacePtr<Interface>* ptr) {
- mojo::MessagePipe pipe;
- ptr->Bind(pipe.handle0.Pass());
- ConnectTo(service_name, pipe.handle1.Pass());
- }
-
// Call this function to resume the navigation when it was deferred
// immediately after receiving response headers.
void ResumeResponseDeferredAtStart(const GlobalRequestID& request_id);
+ // Activates Mojo for this process. Does nothing if Mojo is already activated.
+ void EnsureMojoActivated();
+
protected:
// A proxy for our IPC::Channel that lives on the IO thread (see
// browser_process.h)
diff --git a/content/browser/renderer_host/render_view_host_impl.cc b/content/browser/renderer_host/render_view_host_impl.cc
index e14b364..6b96be6 100644
--- a/content/browser/renderer_host/render_view_host_impl.cc
+++ b/content/browser/renderer_host/render_view_host_impl.cc
@@ -48,11 +48,9 @@
#include "content/common/frame_messages.h"
#include "content/common/input_messages.h"
#include "content/common/inter_process_time_ticks_converter.h"
-#include "content/common/mojo/mojo_service_names.h"
#include "content/common/speech_recognition_messages.h"
#include "content/common/swapped_out_messages.h"
#include "content/common/view_messages.h"
-#include "content/common/web_ui_setup.mojom.h"
#include "content/public/browser/ax_event_notification_details.h"
#include "content/public/browser/browser_accessibility_state.h"
#include "content/public/browser/browser_context.h"
@@ -656,27 +654,6 @@ void RenderViewHostImpl::SetHasPendingCrossSiteRequest(
GetProcess()->GetID(), GetRoutingID(), has_pending_request);
}
-void RenderViewHostImpl::SetWebUIHandle(mojo::ScopedMessagePipeHandle handle) {
- // Never grant any bindings to browser plugin guests.
- if (GetProcess()->IsIsolatedGuest()) {
- NOTREACHED() << "Never grant bindings to a guest process.";
- return;
- }
-
- if ((enabled_bindings_ & BINDINGS_POLICY_WEB_UI) == 0) {
- NOTREACHED() << "You must grant bindings before setting the handle";
- return;
- }
-
- DCHECK(renderer_initialized_);
-
- WebUISetupPtr web_ui_setup;
- static_cast<RenderProcessHostImpl*>(GetProcess())->ConnectTo(
- kRendererService_WebUISetup, &web_ui_setup);
-
- web_ui_setup->SetWebUIHandle(GetRoutingID(), handle.Pass());
-}
-
#if defined(OS_ANDROID)
void RenderViewHostImpl::ActivateNearestFindResult(int request_id,
float x,
diff --git a/content/browser/renderer_host/render_view_host_impl.h b/content/browser/renderer_host/render_view_host_impl.h
index 5f7344f..204d31c 100644
--- a/content/browser/renderer_host/render_view_host_impl.h
+++ b/content/browser/renderer_host/render_view_host_impl.h
@@ -209,7 +209,6 @@ class CONTENT_EXPORT RenderViewHostImpl
const WebPreferences& prefs) OVERRIDE;
virtual void GetAudioOutputControllers(
const GetAudioOutputControllersCallback& callback) const OVERRIDE;
- virtual void SetWebUIHandle(mojo::ScopedMessagePipeHandle handle) OVERRIDE;
virtual void SelectWordAroundCaret() OVERRIDE;
#if defined(OS_ANDROID)
diff --git a/content/browser/webui/web_ui_mojo_browsertest.cc b/content/browser/webui/web_ui_mojo_browsertest.cc
index 5007a8d..6261316 100644
--- a/content/browser/webui/web_ui_mojo_browsertest.cc
+++ b/content/browser/webui/web_ui_mojo_browsertest.cc
@@ -12,15 +12,19 @@
#include "base/strings/string_util.h"
#include "content/browser/webui/web_ui_controller_factory_registry.h"
#include "content/public/browser/browser_context.h"
+#include "content/public/browser/render_frame_host.h"
+#include "content/public/browser/render_process_host.h"
#include "content/public/browser/render_view_host.h"
#include "content/public/browser/web_contents.h"
#include "content/public/browser/web_ui_controller.h"
#include "content/public/browser/web_ui_data_source.h"
#include "content/public/common/content_paths.h"
#include "content/public/common/content_switches.h"
+#include "content/public/common/service_registry.h"
#include "content/public/common/url_utils.h"
#include "content/public/test/content_browser_test.h"
#include "content/public/test/content_browser_test_utils.h"
+#include "content/shell/browser/shell.h"
#include "content/test/data/web_ui_test_mojo_bindings.mojom.h"
#include "grit/content_resources.h"
#include "mojo/common/test/test_utils.h"
@@ -125,13 +129,18 @@ class PingTestWebUIController : public TestWebUIController {
PingTestWebUIController(WebUI* web_ui, base::RunLoop* run_loop)
: TestWebUIController(web_ui, run_loop) {
}
+ virtual ~PingTestWebUIController() {}
// WebUIController overrides:
virtual void RenderViewCreated(RenderViewHost* render_view_host) OVERRIDE {
- mojo::MessagePipe pipe;
- browser_target_.reset(
- new PingBrowserTargetImpl(pipe.handle0.Pass(), run_loop_));
- render_view_host->SetWebUIHandle(pipe.handle1.Pass());
+ render_view_host->GetMainFrame()->GetServiceRegistry()->AddService(
+ "webui_controller",
+ base::Bind(&PingTestWebUIController::CreateHandler,
+ base::Unretained(this)));
+ }
+
+ void CreateHandler(mojo::ScopedMessagePipeHandle handle) {
+ browser_target_.reset(new PingBrowserTargetImpl(handle.Pass(), run_loop_));
}
private:
@@ -212,6 +221,19 @@ IN_PROC_BROWSER_TEST_F(WebUIMojoTest, EndToEndPing) {
// RunLoop is quit when message received from page.
run_loop.Run();
EXPECT_TRUE(got_message);
+
+ // Check that a second render frame in the same renderer process works
+ // correctly.
+ Shell* other_shell = CreateBrowser();
+ got_message = false;
+ base::RunLoop other_run_loop;
+ factory()->set_run_loop(&other_run_loop);
+ NavigateToURL(other_shell, test_url);
+ // RunLoop is quit when message received from page.
+ other_run_loop.Run();
+ EXPECT_TRUE(got_message);
+ EXPECT_EQ(shell()->web_contents()->GetRenderProcessHost(),
+ other_shell->web_contents()->GetRenderProcessHost());
}
} // namespace
diff --git a/content/browser/mojo/mojo_application_host.h b/content/browser/mojo/mojo_application_host.h
index f591326..ae57104 100644
--- a/content/browser/mojo/mojo_application_host.h
+++ b/content/browser/mojo/mojo_application_host.h
@@ -6,9 +6,9 @@
#define CONTENT_BROWSER_MOJO_MOJO_APPLICATION_HOST_H_
#include "base/process/process_handle.h"
+#include "content/common/mojo/service_registry_impl.h"
#include "mojo/embedder/channel_init.h"
#include "mojo/embedder/scoped_platform_handle.h"
-#include "mojo/public/interfaces/service_provider/service_provider.mojom.h"
namespace IPC {
class Sender;
@@ -19,49 +19,31 @@ namespace content {
// MojoApplicationHost represents the code needed on the browser side to setup
// a child process as a Mojo application via Chrome IPC. The child process
// should use MojoApplication to handle messages generated by an instance of
// MojoApplicationHost. MojoApplicationHost makes the mojo::ShellClient
// interface available so that child-provided services can be invoked.
class MojoApplicationHost {
public:
MojoApplicationHost();
virtual ~MojoApplicationHost();
// Two-phase initialization:
// 1- Init makes the shell_client() available synchronously.
// 2- Activate establishes the actual connection to the peer process.
bool Init();
bool Activate(IPC::Sender* sender, base::ProcessHandle process_handle);
bool did_activate() const { return did_activate_; }
- mojo::ServiceProvider* service_provider() {
- DCHECK(child_service_provider_.get());
- return child_service_provider_->client();
- }
+ ServiceRegistry* service_registry() { return &service_registry_; }
private:
- class ServiceProviderImpl
- : public mojo::InterfaceImpl<mojo::ServiceProvider> {
- public:
- virtual void OnConnectionError() OVERRIDE {
- // TODO(darin): How should we handle this error?
- }
-
- // mojo::ServiceProvider methods:
- virtual void ConnectToService(
- const mojo::String& service_url,
- const mojo::String& service_name,
- mojo::ScopedMessagePipeHandle handle,
- const mojo::String& requestor_url) OVERRIDE;
- };
-
mojo::embedder::ChannelInit channel_init_;
mojo::embedder::ScopedPlatformHandle client_handle_;
- scoped_ptr<ServiceProviderImpl> child_service_provider_;
-
bool did_activate_;
+ ServiceRegistryImpl service_registry_;
+
DISALLOW_COPY_AND_ASSIGN(MojoApplicationHost);
};
@@ -19,49 +19,31 @@ namespace content {
// a child process as a Mojo application via Chrome IPC. The child process
// should use MojoApplication to handle messages generated by an instance of
-// MojoApplicationHost. MojoApplicationHost makes the mojo::ShellClient
-// interface available so that child-provided services can be invoked.
+// MojoApplicationHost. MojoApplicationHost makes the ServiceRegistry interface
+// available so that child-provided services can be invoked.
class MojoApplicationHost {
public:
@@ -19,49 +19,31 @@ namespace content {
virtual ~MojoApplicationHost();
// Two-phase initialization:
- // 1- Init makes the shell_client() available synchronously.
+ // 1- Init makes service_registry() available synchronously.
// 2- Activate establishes the actual connection to the peer process.
bool Init();